/* fileformat.c
 * File format funtions: uncompres, decode, ...
 *
 * $Id:  $
 *
 * Xplico - Internet Traffic Decoder
 * By Gianluca Costa <g.costa@xplico.org>
 * Copyright 2009 Gianluca Costa & Andrea de Franceschi. Web: www.xplico.org
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <pthread.h>
#include <fcntl.h>
#include <zlib.h>

#include "fileformat.h"
#include "istypes.h"
#include "dmemory.h"
#include "log.h"

/* max malloc limit */
#define  FF_MALLOC_LIMIT (10*1024*1024)
#define  FF_BUFFER_DIM   (1024*1024)

int FFormatUncompress(const char *encoding, const char *file_in,  const char *file_out)
{
    bool decode;
    z_stream zbuff;
    unsigned char *buff;
    unsigned char *raw;
    ssize_t size;
    int ret;
    size_t wsize;
    struct stat info_file;
    FILE *fp;
    
    if (encoding != NULL) {
        decode = FALSE;
        memset(&zbuff, 0, sizeof(z_stream));
        if (strcasecmp(encoding, "gzip") == 0) {
            if (inflateInit2(&zbuff, 47) == Z_OK) {
                decode = TRUE;
            }
        }
        else if (strcasecmp(encoding, "deflate") == 0) {
            if (inflateInit2(&zbuff, -15) == Z_OK) {
                decode = TRUE;
            }
        }
        if (decode && stat(file_in, &info_file) == 0 && info_file.st_size < FF_MALLOC_LIMIT) {
            raw = xmalloc(info_file.st_size);
            fp = fopen(file_in, "r");
            zbuff.avail_in = fread(raw, 1, info_file.st_size, fp);
            fclose(fp);
            zbuff.next_in = raw;
            size = 10240;
            buff = xmalloc(size);
            if (buff != NULL) {
                fp = fopen(file_out, "w");
                do {
                    zbuff.next_out = buff;
                    zbuff.avail_out = size;
                    ret = inflate(&zbuff, Z_SYNC_FLUSH);
                    if (ret == Z_OK || ret == Z_STREAM_END) {
                        wsize = size - zbuff.avail_out;
                        if (fp != NULL)
                            fwrite(buff, 1, wsize, fp);
			if (ret == Z_STREAM_END) {
                            inflateEnd(&zbuff);
                            break;
			}
                    }
                    else {
                        inflateEnd(&zbuff);
                        xfree(buff);
                        xfree(raw);
                        rename(file_in, file_out);
                        
                        return -1;
                    }
                } while (zbuff.avail_in);
                if (fp != NULL)
                    fclose(fp);
                xfree(buff);
            }
            else {
                LogPrintf(LV_ERROR, "No memory");
            }
            xfree(raw);
        }
        else {
            rename(file_in, file_out);
        }
    }
    
    return 0;
}


int FFormatCopy(char *old, char *new)
{
    char *buff;
    FILE *in, *out;
    size_t len;
    int ret = 0;

    /* copy */        
    in = fopen(old, "r");
    if (in != NULL) {
        out = fopen(new, "w");
        if (out != NULL) {
            buff = xmalloc(FF_BUFFER_DIM);
            if (buff != NULL) {
                while ((len = fread(buff, 1, FF_BUFFER_DIM, in)) != 0)
                    fwrite(buff, 1, len, out);
                xfree(buff);
            }
            fclose(out);
        }
        else {
            LogPrintf(LV_WARNING, "Unable to open file (%s)", new);
            ret = -1;
        }
        fclose(in);
    }
    else {
        LogPrintf(LV_WARNING, "Unable to open file (%s)", old);
        ret = -1;
    }

    return ret;
}


void f_format_link(void)
{
    
}
