/* dema.c
 *
 * Decoding Manager daemon 
 *
 * $Id: $
 *
 * Xplico System
 * By Gianluca Costa <g.costa@xplico.org>
 * Copyright 2007-2010 Gianluca Costa & Andrea de Franceschi. Web: www.xplico.org
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <signal.h>
#include <sys/wait.h>
#include <signal.h>

#include "dema.h"
#include "session_decoding.h"
#include "dbinterface.h"
#include "captime.h"
#include "version.h"

static volatile bool terminate;

static void Usage(char *name)
{
    printf("\n");
    printf("usage: %s {-c <config_file> | {-d <pol_root_dir> -b %s|%s}} [-h]\n", name, DB_T_MYSQL, DB_T_SQLITE);
    printf("\t-c config file\n");
    printf("\t-d pols root dir\n");
    printf("\t-b DB type (mysql or sqlite)\n");
    printf("\t-h this help\n");
    printf("\n");
}


static void DemaDecInit(podec *dec)
{
    memset(dec, 0, sizeof(podec));
    dec->pol_id = -1;
    dec->sol_id = -1;
    dec->run = FALSE;
    dec->end = FALSE;
    dec->name[0] = '\0';
    dec->size = 0;
    dec->filenum = 0;
    dec->rt = FALSE;
}


int DemaHash(const char *path_src, char *md5, char *sha1)
{
    char cmd[2*DM_FILENAME_PATH];
    char buffer[2*DM_HASH_STR];
    char dummy[DM_HASH_STR];
    FILE *fp;
    int res, ret = 0;
    
    /* run md5sum and sha1sum */
    sprintf(cmd, "md5sum %s > /tmp/dema_hash.txt; sha1sum %s >> /tmp/dema_hash.txt", path_src, path_src);
    system(cmd);
    fp = fopen("/tmp/dema_hash.txt", "r");
    if (fp != NULL) {
        if (fgets(buffer, 2*DM_HASH_STR, fp) != NULL) {
            /* md5 */
            res = sscanf(buffer, "%s %s", md5, dummy);
            if (res != 2) {
                ret = -1;
            }

            /* sha1 */
            if (fgets(buffer, 2*DM_HASH_STR, fp) != NULL) {
                res = sscanf(buffer, "%s %s", sha1, dummy);
                if (res != 2) {
                    ret = -1;
                }
            }
            else {
                ret = -1;
            }
        }
        else {
            ret = -1;
        }
        
        fclose(fp);
    }
    else {
        ret = -1;
    }

    remove("/tmp/dema_hash.txt");

    return ret;
}


static void DemaSigTerm(int sig)
{
    terminate = TRUE;
}


static int DemaLoop(dbtype db_t, char *root)
{
    podec *pd_tbl;
    int dim, num, i, j;
    struct stat info;
    char *filename, *tmp;
    char dir[DM_FILENAME_PATH];
    char path_src[DM_FILENAME_PATH];
    char path_dst[DM_FILENAME_PATH];
    char cmd[DM_FILENAME_PATH];
    char interf[DM_FILENAME_PATH];
    int nxt_ses;
    pid_t chld;
    cap_time *ctime;
    char sha1[DM_HASH_STR];
    char md5[DM_HASH_STR];
    FILE *fp;
    short session_rm;
    
    pd_tbl = malloc(sizeof(podec)*DM_TBL_ADD);
    for (i=0; i!=DM_TBL_ADD; i++) {
        DemaDecInit(&(pd_tbl[i]));
    }
    dim = DM_TBL_ADD;
    num = 0;
    session_rm = 0;
    
    /* main loop */
    while (1) {
        /* find new pol */
        i = SeDeFind(root, pd_tbl, dim);
        if (i > 0) {
            num += i;
            if (dim-num < DM_TBL_ADD) {
                pd_tbl = realloc(pd_tbl, sizeof(podec)*(dim+DM_TBL_ADD));
                memset(pd_tbl+dim, -1, sizeof(podec)*DM_TBL_ADD);
                for (i=dim; i!= dim+DM_TBL_ADD; i++) {
                    DemaDecInit(&(pd_tbl[i]));
                }
                dim += DM_TBL_ADD;
            }
        }
        
        /* check file to decode */
        for (i=0, j=0; j<num; i++) {
            if (pd_tbl[i].pol_id == -1)
                continue;
            j++;
            /* if not rt decoding */
            if (pd_tbl[i].rt == TRUE)
                continue;
            filename = SeDeFile(root, pd_tbl[i].pol_id, pd_tbl[i].sol_id);
            if (filename != NULL && pd_tbl[i].end == FALSE) {
                /* check file name */
                if (strcmp(filename, pd_tbl[i].name) != 0) {
                    pd_tbl[i].size = 0;
                    strcpy(pd_tbl[i].name, filename);
                }

                sprintf(dir, DM_NEW_DIR, root, pd_tbl[i].pol_id, pd_tbl[i].sol_id);
                sprintf(path_src, "%s/%s", dir, filename);

                if (stat(path_src, &info) == 0) {
                    if (pd_tbl[i].size == info.st_size) {
                        /* capture time update */
                        ctime = CapTime(path_src);
                        if (ctime != NULL) {
                            /* checksum/hash */
                            if (DemaHash(path_src, md5, sha1) != 0) {
                                /* empty hash */
                                md5[0] = '\0';
                                sha1[0] = '\0';
                            }

                            /* insert/update data in DB */
                            DBIntCapTime(pd_tbl[i].pol_id, pd_tbl[i].sol_id, ctime);
                            DBIntInputPcap(pd_tbl[i].pol_id, pd_tbl[i].sol_id, ctime, pd_tbl[i].size, pd_tbl[i].name, md5, sha1);
                        }
                        else {
                            printf("Error: incorrect capture file %s\n", filename);
                            /* free */
                            pd_tbl[i].name[0] = '\0';
                            pd_tbl[i].size = 0;

                            /* remove file */
                            remove(path_src);
                            continue;
                        }
                        /* free */
                        pd_tbl[i].name[0] = '\0';
                        pd_tbl[i].size = 0;

                        /* move file in raw */
                        sprintf(dir, DM_RAW_DIR, root, pd_tbl[i].pol_id, pd_tbl[i].sol_id);
                        sprintf(path_dst, "%s/%s", dir, filename);
                        rename(path_src, path_dst);

                        /* link file in decode */
                        sprintf(path_src, "%s/%s", dir, filename);
                        sprintf(dir, DM_DECOD_DIR, root, pd_tbl[i].pol_id, pd_tbl[i].sol_id);
                        sprintf(path_dst, "%s/%s", dir, filename);
                        link(path_src, path_dst);
                        pd_tbl[i].filenum++;

                        /* start decoding */
                        if (pd_tbl[i].run == FALSE) {
                            if (SeDeStart(db_t, root, pd_tbl[i].pol_id, pd_tbl[i].sol_id, &pd_tbl[i].pid, FALSE, NULL) == 0) {
                                /* change status in to db */
                                DBIntDecStart(pd_tbl[i].pol_id, pd_tbl[i].sol_id);
                                pd_tbl[i].run = TRUE;
                                pd_tbl[i].end_to = -1;
                            }
                            else {
                                printf("Applications executions error\n");
                                exit(-1);
                            }
                        }
                        
                        /* exist new file */
                        filename = SeDeFile(root, pd_tbl[i].pol_id, pd_tbl[i].sol_id);
                        if (filename != NULL) {
                            strcpy(pd_tbl[i].name, filename);
                            sprintf(dir, DM_NEW_DIR, root, pd_tbl[i].pol_id, pd_tbl[i].sol_id);
                            sprintf(path_src, "%s/%s", dir, filename);
                            if (stat(path_src, &info) == 0) {
                                pd_tbl[i].size = info.st_size;
                            }
                        }
                    }
                    else {
                        pd_tbl[i].size = info.st_size;
                    }
                }
                else {
                    perror("");
                }
            }
            else if (pd_tbl[i].end == FALSE && pd_tbl[i].run == TRUE) {
                /* it isn't a new file */
                SeDeEnd(root, pd_tbl[i].pol_id, pd_tbl[i].sol_id, &pd_tbl[i].pid);
                pd_tbl[i].end = TRUE;
            }
        }

        /* examine sessions in the pol */
        for (i=0, j=0; j<num; i++) {
            if (pd_tbl[i].pol_id == -1)
                continue;
            j++;
            if (pd_tbl[i].run == FALSE && pd_tbl[i].size == 0) {
                nxt_ses = SeDeNextSession(root, pd_tbl[i].pol_id, pd_tbl[i].sol_id);
                if (nxt_ses != -1) {
                    printf("Created new sol with id:%i\n", nxt_ses);
                    pd_tbl[i].sol_id = nxt_ses;
                }
            }
        }
        
        /* check case and session deletion */
        for (i=0, j=0; j<num; i++) {
            if (pd_tbl[i].pol_id == -1)
                continue;
            j++;
            sprintf(path_src, DM_POL_DIR"/%s", root, pd_tbl[i].pol_id, DM_DELETE_CASE);
            if (stat(path_src, &info) == 0) {
                /* delete pol (case) */
                SeDeKill(pd_tbl, i);
                /* delete records in DB */
                DBIntDeletePol(pd_tbl[i].pol_id);
                remove(path_src); /* this frees the XI */
                /* delete files */
                sprintf(cmd, "rm -rf "DM_POL_DIR, root, pd_tbl[i].pol_id);
                system(cmd);
                DemaDecInit(&(pd_tbl[i]));
                num--;
            }
            
            /* every DM_ERASE_SESSSION tick we erase all session not used */
            if (session_rm == DM_ERASE_SESSION) {
                /* delete sol records in DB */
                DBIntDeleteSol(pd_tbl[i].pol_id, -1);
                /* delete files of SOL */
                sprintf(cmd, "rm -rf "DM_POL_DIR"/"DM_DELETE_SESSION, root, pd_tbl[i].pol_id);
                system(cmd);
            }
        }
        if (session_rm > DM_ERASE_SESSION)
            session_rm = 0;

        /* check realtime case -start/stop- */
        for (i=0, j=0; j<num; i++) {
            if (pd_tbl[i].pol_id == -1)
                continue;
            j++;
            
            /* stop */
            sprintf(path_dst, DM_POL_DIR"/%s", root, pd_tbl[i].pol_id, DM_RT_STOP_FILE);
            if (stat(path_dst, &info) == 0) {
                /* stop rt acquisition */
                SeDeEnd(root, pd_tbl[i].pol_id, pd_tbl[i].sol_id, &pd_tbl[i].pid);
                pd_tbl[i].end = TRUE;
                pd_tbl[i].rt = FALSE;
                sprintf(path_src, DM_POL_DIR"/%s", root, pd_tbl[i].pol_id, DM_RT_START_FILE);
                remove(path_src);
                remove(path_dst);
            }
            if (pd_tbl[i].run == TRUE)
                continue;
            
            /* start */
            sprintf(path_src, DM_POL_DIR"/%s", root, pd_tbl[i].pol_id, DM_RT_START_FILE);
            if (stat(path_src, &info) == 0) {
                /* netework interface */
                fp = fopen(path_src, "r");
                if (fp != NULL) {
                    /* network intrface */
                    if (fgets(interf, DM_FILENAME_PATH, fp) != NULL) {
                        fclose(fp);
                        if ((tmp = strchr(interf, '\r')) != NULL) {
                            tmp[0] = '\0';
                        }
                        if ((tmp = strchr(interf, ' ')) != NULL) {
                            tmp[0] = '\0';
                        }
                        if ((tmp = strchr(interf, '\n')) != NULL) {
                            tmp[0] = '\0';
                        }
                        
                        /* start rt acquisition */
                        if (SeDeStart(db_t, root, pd_tbl[i].pol_id, pd_tbl[i].sol_id, &pd_tbl[i].pid, TRUE, interf) == 0) {
                            /* change status in to db */
                            DBIntDecStart(pd_tbl[i].pol_id, pd_tbl[i].sol_id);
                            pd_tbl[i].run = TRUE;
                            pd_tbl[i].rt = TRUE;
                            pd_tbl[i].end_to = -1;
                        }
                        else {
                            remove(path_src);
                            printf("Applications executions error\n");
                            exit(-1);
                        }
                    }
                    else {
                        remove(path_src);
                        printf("Interface error\n");
                    }
                }
                else {
                    remove(path_src);
                }
            }
        }

        /* check end timeout */
        for (i=0, j=0; j<num; i++) {
            if (pd_tbl[i].pol_id == -1)
                continue;
            j++;
            if (pd_tbl[i].end == TRUE && pd_tbl[i].run == TRUE) {
                if (pd_tbl[i].end_to == 0) {
                    /* force kill all task */
                    SeDeKill(pd_tbl, i);
                    pd_tbl[i].end_to = -1;
                }
                else if (pd_tbl[i].end_to != -1)
                    pd_tbl[i].end_to--;
            }
        }

        /* check termination */
        if (terminate == TRUE) {
            for (i=0; i!=dim; i++) {
                SeDeKill(pd_tbl, i);
            }
            system("killall xplico");
            break; /* exit from main cicle */
        }

        /* check process termination */
        do {
            chld = waitpid(0, NULL, WNOHANG);
            if (chld > 0) {
                for (i=0, j=0; j<num; i++) {
                    if (pd_tbl[i].pol_id == -1)
                        continue;
                    j++;
                    if (pd_tbl[i].run == TRUE) {
                        if (SeDeRun(&pd_tbl[i].pid, chld) == 0) {
                            if (pd_tbl[i].end) {
                                pd_tbl[i].end_to = DM_END_TO;
                                pd_tbl[i].pid.tot--;
                                if (pd_tbl[i].pid.tot == 0) {
                                    pd_tbl[i].run = FALSE;
                                    pd_tbl[i].end = FALSE;
                                    pd_tbl[i].rt = FALSE;
                                    /* change status in to db */
                                    DBIntDecEnd(pd_tbl[i].pol_id, pd_tbl[i].sol_id);
                                }
                            }
                            else {
                                /* force end with kill */
                                pd_tbl[i].pid.tot--;
                                pd_tbl[i].end = TRUE;
                                SeDeKill(pd_tbl, i);
                                printf("Xplico or a Manipulator is dead!\n");
                            }
                        }
                    }
                }
            }
            else {
                chld = 0;
            }
        } while (chld);
        
        /* wait */
        sleep(1);
        session_rm++;
    }
    
    /* free memory */
    free(pd_tbl);
    
    return 0;
}


int main(int argc, char *argv[])
{
    char c;
    char config_file[512];
    char root_dir[512];
    char db_type[512];
    dbtype db_t;
    FILE *run;
    extern char *optarg;
    extern int optind, optopt;
    
    printf("dema v%d.%d.%d\n", DEMA_VER_MAG, DEMA_VER_MIN, DEMA_VER_REV);
    printf("%s\n", DEMA_CR);

    config_file[0] = '\0';
    root_dir[0] = '\0';
    db_type[0] = '\0';
    while ((c = getopt(argc, argv, "c:d:b:h")) != -1) {
        switch(c) {
        case 'c':
            sprintf(config_file, "%s", optarg);
            break;

        case 'd':
            sprintf(root_dir, "%s", optarg);
            break;

        case 'b':
            sprintf(db_type, "%s", optarg);
            break;

        case 'h':
            Usage(argv[0]);
            return 0;
            break;

        case '?':
            printf("Error: unrecognized option: -%c\n", optopt);
            Usage(argv[0]);
            exit(-1);
            break;
        }
    }

    /* db type */
    if (strcmp(db_type, DB_T_MYSQL) == 0) {
        db_t = DB_MYSQL;
    }
    else if (strcmp(db_type, DB_T_SQLITE) == 0) {
        db_t = DB_SQLITE;
    }
    else if (strcmp(db_type, DB_T_POSTGRES) == 0) {
        db_t = DB_POSTGRES;
    }
    else {
        Usage(argv[0]);

        return 0;
    }

    if (root_dir[0] != '\0') {
        /* init db connection */
        if (DBIntInit(db_t) == -1)
            printf("Error: DB interface\n");
        
        /* daemon */
        daemon(1,1);

        /* kill all xplico running */
        system("killall xplico");

        /* sigterm function */
        terminate = FALSE;
        signal(SIGTERM, DemaSigTerm);

        run = fopen(DM_DEMA_RUN, "w+");
        if (run != NULL) {
            fprintf(run, "%i\n", getpid());
            fclose(run);
            DemaLoop(db_t, root_dir);
        }
        /* close db connection */
        DBIntClose();
    }
    else {
        Usage(argv[0]);
    }

    return 0;
}
