/* dbinterface.h
 *
 * $Id: dbinterface.h,v 1.1 2007/09/08 07:11:52 costa Exp $
 *
 * Xplico System
 * By Gianluca Costa <g.costa@xplico.org>
 * Copyright 2007-2010 Gianluca Costa & Andrea de Franceschi. Web: www.xplico.org
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


#ifndef __DBINTERFACE_H__
#define __DBINTERFACE_H__

#include "captime.h"

#ifndef DB_SQLITE2
# error "Makefile error: sqlite version is not defined"
#endif

/* common */
#define DBINT_DIR_PATH            "/opt/xplico/pol_%d/sol_%d"
#define DBINT_QUERY_DIM           10240
#define DBINT_QUERY_START_DEC     "UPDATE sols SET status='DECODING' WHERE id='%d';"
#define DBINT_QUERY_END_DEC       "UPDATE sols SET status='DECODING COMPLETED' WHERE id='%d';"
#define DBINT_QUERY_DELETE_POL    "DELETE FROM pols WHERE id='%d';"
#define DBINT_QUERY_DELETE_SOL    "DELETE FROM sols WHERE id='%d';"
#define DBINT_QUERY_DELETE_RM_SOL "DELETE FROM sols WHERE rm='1' AND pol_id='%d';"

/* mysql */
#define DBINT_HOST_NAME       "localhost"        /* host to connect to (default = localhost) */
#define DBINT_USER_NAME       "xplico"           /* user name (default = your login name) */
#define DBINT_PASSWORD        "Y8seydq9pWERwK22" /* password (default = none) */
#define DBINT_DB_NAME         "xplico"           /* database to use (default = none) */
#define DBINT_1_QUERY_START_T_TEMPLATE  "UPDATE sols SET start_time=FROM_UNIXTIME(%lu) WHERE id='%d' AND start_time='0000-00-00 00:00:00';"
#define DBINT_1_QUERY_END_T_TEMPLATE    "UPDATE sols SET end_time=FROM_UNIXTIME(%lu) WHERE id='%d';"
#define DBINT_1_QUERY_INPUT_FILE        "INSERT INTO inputs (pol_id, sol_id, start_time, end_time, data_size, filename, md5, sha1) VALUES (%i, %i, FROM_UNIXTIME(%lu), FROM_UNIXTIME(%lu), %lu, '%s', '%s', '%s');"

#if DB_SQLITE2 == 0

/* sqlite 2 */
#define DBINT_DB_FILE         "/opt/xplico/xplico.db" /* DB file path */
#define DBINT_2_QUERY_START_T_TEMPLATE  "UPDATE sols SET start_time=DATETIME('NOW', 'LOCALTIME', '-%lu seconds') WHERE id='%d' AND start_time='0000-00-00 00:00:00';"
#define DBINT_2_QUERY_END_T_TEMPLATE    "UPDATE sols SET end_time=DATETIME('NOW', 'LOCALTIME', '-%lu seconds') WHERE id='%d';"
#define DBINT_2_QUERY_INPUT_FILE        "INSERT INTO inputs (pol_id, sol_id, start_time, end_time, data_size, filename, md5, sha1) VALUES (%i, %i, DATETIME('NOW', 'LOCALTIME', '-%lu seconds'), DATETIME('NOW', 'LOCALTIME', '-%lu seconds'), %lu, '%s', '%s', '%s');"

#else

/* sqlite 3 */
#define DBINT_DB_FILE         "/opt/xplico/xplico.db" /* DB file path */
#define DBINT_2_QUERY_START_T_TEMPLATE  "UPDATE sols SET start_time=DATETIME(%lu, 'unixepoch') WHERE id='%d' AND start_time='0000-00-00 00:00:00';"
#define DBINT_2_QUERY_END_T_TEMPLATE    "UPDATE sols SET end_time=DATETIME(%lu, 'unixepoch') WHERE id='%d';"
#define DBINT_2_QUERY_INPUT_FILE        "INSERT INTO inputs (pol_id, sol_id, start_time, end_time, data_size, filename, md5, sha1) VALUES (%i, %i, DATETIME(%lu, 'unixepoch'), DATETIME(%lu, 'unixepoch'), %lu, '%s', '%s', '%s');"

#endif /* DB_SQLITE2 */

typedef enum {
    DB_MYSQL,
    DB_SQLITE,  /* sqlite 3 or 2*/
    DB_POSTGRES
} dbtype;


int DBIntInit(dbtype type);
int DBIntClose(void);
int DBIntCapTime(int pol_id, int sol_id, cap_time *ctime);
int DBIntDecStart(int pol_id, int sol_id);
int DBIntDecEnd(int pol_id, int sol_id);
int DBIntInputPcap(int pol_id, int sol_id, cap_time *ctime, unsigned long size,
                   const char *name, const char *md5, const char *sha1);
int DBIntDeletePol(int pol_id);
int DBIntDeleteSol(int pol_id, int sol_id);

#endif /* __DBINTERFACE_H__ */
